function W = dtwtmatrix(Lo_D,Hi_D,J,n)
%  Funkce dtwtmatrix pro tvorbu DTWT analyzujc matice, funguje i pro
%  biortogonln wavelety.
%  Funkce vznikla jako doprovodn k dokumentu
%  Rajmic, P., Pra, Z. Podrobn studie algoritmu pro vpoet waveletov transformace v diskrtnm ase
%
%% Vznam vstupnch parametr: 
%    Lo_D: impulzn charakteristika analyzujc doln propusti
%    Hi_D: impulzn charakteristika analyzujc horn propusti
%       J: hloubka dekompozice 
%       n: dlka transformace (tj. dlka vektoru, kter m bt transformovn)
%          mus platit n=2^(J+k) pro njak k z mnoiny {0,1,2...}
%                          
%% Vznam vstupnch parametr: 
%       W: analyzujc matice
%% Poznmky:
%       - Transformaci vektoru potom sta provst jako nsoben: y = W*x
%       - Syntetizujc matici je mon zskat jako inv(W)
%
%% Pklad pouit:
%       [Lo_D,Hi_D,Lo_R,Hi_R] = w_filters('db3');
%       W = dtwtmatrix(Lo_D,Hi_D,3,2^6);
%       y = W*signal  %signal je rozmeru 2^6 x 1

%  (c) 2010 Zdenk Pra, Pavel Rajmic


%% Overeni korektni delky transformace
N = n;
NN = N;
while 1
  NN = NN/2;
  if NN < 1
      error('Dlka transformace nen mocnina dvou');break;
  end
  if NN==1
      break;
  end
end

if N<2^J
    error('Dlka transformace je men ne 2^J');
end


A = zeros(N);
Nhalf = floor(N/2);

LO = fliplr(Lo_D(:)');
HI = fliplr(Hi_D(:)');

HI_N=length(HI);
LO_N=length(LO);

H = zeros(N,Nhalf);
G = zeros(N,Nhalf);

bufLen = N;
Htemp = eye(N);

for d=1:J
    
    H = zeros(N,Nhalf); %horn
    G = zeros(N,Nhalf); %doln polovina matice W_d
    
    %% vpoet posun impulznch charakteristik pro matici G_d
    for i = 0:Nhalf-1
        temp = zeros(N,1);
        if HI_N > N
            Ndiv = floor(HI_N/N);
            Nmodulo = mod(HI_N,N);
            for I = 0:Ndiv-1
                temp = temp + HI(I*N+1:(I+1)*N)';
            end
            if Nmodulo>0
                temp(1:Nmodulo) = temp(1:Nmodulo) + HI(HI_N-Nmodulo+1:HI_N)';
            end
        else
            temp(1:HI_N) = HI';
        end
        
        temp = circshift(temp,i*2-(HI_N-1)+HI_N/2);
        G(:,i+1) = temp; % vypoten matice G_d
    end
    
    %% vpoet posun impulznch charakteristik pro matici H_d
    for i = 0:Nhalf-1
        temp = zeros(N,1);
        if LO_N > N
            Ndiv = floor(LO_N/N);
            Nmodulo = mod(LO_N,N);
            for I = 0:Ndiv-1
                temp = temp + LO(I*N+1:(I+1)*N)';
            end
            if Nmodulo>0
                temp(1:Nmodulo) = temp(1:Nmodulo) + LO(LO_N-Nmodulo+1:LO_N)';
            end
        else
            temp(1:LO_N) = LO';
        end
        temp = circshift(temp,i*2-(LO_N-1)+LO_N/2);
        H(:,i+1) = temp; % vypoten matice H_d
        
    end

A(:,Nhalf+1:N) = Htemp*G; % nsoben dlch matic podle vztahu (14) z dokumentu. 
Htemp = Htemp*H;

Nhalf = floor(Nhalf/2);
N = N/2;
end

A(:,1:N) = Htemp;
W=A';
